@@ -44,7 +44,7 @@
 #' data_extraction_sheet(measure = "md", extension = "data.frame")
 data_extraction_sheet <- function(measure = c("d", "g", "md", "or", "rr", "nnt",
                                               "r", "z", "logvr", "logcvr", "irr"),
-                                  type_of_measure = c("natural+converted", "natural"),
+                                  type_of_measure = c("natural", "natural+converted"),
                                   name = "mcv_data_extraction",
                                   extension = c("data.frame", ".txt", ".csv", ".xlsx"),
                                   verbose = TRUE
